"""
maskSLIC: Modified from scikit-image slic method

Original code (C) scikit-image
Modification (C) 2016-2019 Benjamin Irving

See LICENSE.txt for more details
"""

# coding=utf-8
from __future__ import division, absolute_import, unicode_literals, print_function

import warnings
import collections as coll

import numpy as np

from scipy import ndimage as ndi
from scipy.ndimage.morphology import distance_transform_edt
from scipy.ndimage.filters import gaussian_filter

from skimage.util import img_as_float, regular_grid
from skimage.color import rgb2lab

from ._slic import _slic_cython, _enforce_label_connectivity_cython
from .processing import get_mpd

def place_seed_points(image, mask, n_segments, spacing):
    """
    Method for placing seed points in an ROI

    Note:
    Optimal point placement problem is somewhat related to the k-center problem
     metric facility location (MFL)
     Maxmin facility location
    https://en.wikipedia.org/wiki/Facility_location_problem

    :param image:
    :param mask:
    :param n_segments:
    :param spacing:

    :return:
    """
    segments_z = np.zeros(n_segments, dtype=np.int64)
    segments_y = np.zeros(n_segments, dtype=np.int64)
    segments_x = np.zeros(n_segments, dtype=np.int64)

    m_inv = np.copy(mask)

    # Cropping to bounding box around ROI
    nonzero_x, nonzero_y, nonzero_z = np.nonzero(m_inv)
    bbox_start = [np.min(nonzero_x), np.min(nonzero_y), np.min(nonzero_z)]
    bbox_end = [np.max(nonzero_x), np.max(nonzero_y), np.max(nonzero_z)]
    m_inv = m_inv[bbox_start[0]:bbox_end[0]+1, bbox_start[1]:bbox_end[1]+1, bbox_start[2]:bbox_end[2]+1]

    # SEED STEP 1:  n seeds are placed as far as possible from every other seed and the edge.
    for seg_idx in range(n_segments):

        # Distance transform
        dtrans = distance_transform_edt(m_inv, sampling=spacing)
        dtrans = gaussian_filter(dtrans, sigma=0.1)

        # Use the maximum locations for the first two points
        coords1 = np.nonzero(dtrans == np.max(dtrans))
        segments_z[seg_idx] = coords1[0][0]
        segments_x[seg_idx] = coords1[1][0]
        segments_y[seg_idx] = coords1[2][0]

        # Adding a new point
        m_inv[segments_z[seg_idx], segments_x[seg_idx], segments_y[seg_idx]] = False

    segments_z = segments_z + bbox_start[0]
    segments_x = segments_x + bbox_start[1]
    segments_y = segments_y + bbox_start[2]

    segments_color = np.zeros((segments_z.shape[0], image.shape[3]))
    segments = np.concatenate([segments_z[..., np.newaxis],
                               segments_x[..., np.newaxis],
                               segments_y[..., np.newaxis],
                               segments_color], axis=1)

    segments_z = np.ascontiguousarray(segments_z, dtype=np.int32)
    segments_x = np.ascontiguousarray(segments_x, dtype=np.int32)
    segments_y = np.ascontiguousarray(segments_y, dtype=np.int32)

    out1 = get_mpd(segments_z, segments_x, segments_y)
    step_z, step_x, step_y = out1[0], out1[1], out1[2]

    return segments, step_x, step_y, step_z

def slic(image, n_segments=100, compactness=10., max_iter=10, sigma=0,
         seed_type='grid', spacing=None, multichannel=True, convert2lab=None,
         enforce_connectivity=False, min_size_factor=0.5, max_size_factor=3,
         slic_zero=False, multifeat=False, return_adjacency=False, mask=None,
         return_segments=False, recompute_seeds=False):
    """
    Segments image using k-means clustering in Color-(x,y,z) space.

    :param image: 2D, 3D or 4D ndarray
        Input image, which can be 2D or 3D, and grayscale or multichannel
        (see `multichannel` parameter).
    :param n_segments: int, optional
        The (approximate) number of labels in the segmented output image.
    :param compactness: float, optional
        Balances color proximity and space proximity. Higher values give
        more weight to space proximity, making superpixel shapes more
        square/cubic. In SLICO mode, this is the initial compactness.
        This parameter depends strongly on image contrast and on the
        shapes of objects in the image. We recommend exploring possible
        values on a log scale, e.g., 0.01, 0.1, 1, 10, 100, before
        refining around a chosen value.
    :param max_iter: int, optional
        Maximum number of iterations of k-means.
    :param sigma: float or (3,) array-like of floats, optional
        Width of Gaussian smoothing kernel for pre-processing for each
        dimension of the image. The same sigma is applied to each dimension in
        case of a scalar value. Zero means no smoothing.
        Note, that `sigma` is automatically scaled if it is scalar and a
        manual voxel spacing is provided (see Notes section).
    :param spacing: (3,) array-like of floats, optional
        The voxel spacing along each image dimension. By default, `slic`
        assumes uniform spacing (same voxel resolution along z, y and x).
        This parameter controls the weights of the distances along z, y,
        and x during k-means clustering.
    :param multichannel: bool, optional
        Whether the last axis of the image is to be interpreted as multiple
        channels or another spatial dimension.
    :param convert2lab: bool, optional
        Whether the input should be converted to Lab colorspace prior to
        maskslic. The input image *must* be RGB. Highly recommended.
        This option defaults to ``True`` when ``multichannel=True`` *and*
        ``image.shape[-1] == 3``.
    :param enforce_connectivity: bool, optional
        Whether the generated segments are connected or not
    :param min_size_factor: float, optional
        Proportion of the minimum segment size to be removed with respect
        to the supposed segment size ```depth*width*height/n_segments```
    :param max_size_factor: float, optional
        Proportion of the maximum connected segment size. A value of 3 works
        in most of the cases.
    :param slic_zero: bool, optional
        Run SLIC-zero, the zero-parameter mode of SLIC. [2]_
    :param mask: ndarray of bools or 0s and 1s, optional
        Array of same shape as `image`. Supervoxel analysis will only be performed on points at
        which mask == True

    :return: labels : 2D or 3D array
        Integer mask indicating segment labels.

    Raises
    ------
    ValueError
        If ``convert2lab`` is set to ``True`` but the last array
        dimension is not of length 3.

    Notes
    -----
    * If `sigma > 0`, the image is smoothed using a Gaussian kernel prior to
      maskslic.

    * If `sigma` is scalar and `spacing` is provided, the kernel width is
      divided along each dimension by the spacing. For example, if ``sigma=1``
      and ``spacing=[5, 1, 1]``, the effective `sigma` is ``[0.2, 1, 1]``. This
      ensures sensible smoothing for anisotropic images.

    * The image is rescaled to be in [0, 1] prior to processing.

    * Images of shape (M, N, 3) are interpreted as 2D RGB images by default. To
      interpret them as 3D with the last dimension having length 3, use
      `multichannel=False`.

    References
    ----------
    .. [1] Radhakrishna Achanta, Appu Shaji, Kevin Smith, Aurelien Lucchi,
        Pascal Fua, and Sabine Susstrunk, SLIC Superpixels Compared to
        State-of-the-art Superpixel Methods, TPAMI, May 2012.
    .. [2] http://ivrg.epfl.ch/research/superpixels#SLICO

    Examples
    --------
    >>> from maskslic import slic
    >>> from skimage.data import astronaut
    >>> img = astronaut()
    >>> segments = slic(img, n_segments=100, compactness=10)

    Increasing the compactness parameter yields more square regions:

    >>> segments = slic(img, n_segments=100, compactness=20)

    """
    if slic_zero:
        raise NotImplementedError("Slic zero has not been implemented yet for maskSLIC.")

    if mask is None and seed_type == 'nplace':
        warnings.warn('nplace assignment of seed points should only be used with an ROI. Changing seed type.')
        seed_type = 'grid'

    if seed_type == 'nplace' and recompute_seeds is False:
        warnings.warn('Seeds should be recomputed when seed points are randomly assigned')

    image = img_as_float(image)
    is_2d = False
    if image.ndim == 2:
        # 2D grayscale image
        image = image[np.newaxis, ..., np.newaxis]
        is_2d = True
    elif image.ndim == 3 and multichannel:
        # Make 2D multichannel image 3D with depth = 1
        image = image[np.newaxis, ...]
        is_2d = True
    elif image.ndim == 3 and not multichannel:
        # Add channel as single last dimension
        image = image[..., np.newaxis]

    if spacing is None:
        spacing = np.ones(3)
    elif isinstance(spacing, (list, tuple)):
        spacing = np.array(spacing, dtype=np.double)

    if not isinstance(sigma, coll.Iterable):
        sigma = np.array([sigma, sigma, sigma], dtype=np.double)
        sigma /= spacing.astype(np.double)
    elif isinstance(sigma, (list, tuple)):
        sigma = np.array(sigma, dtype=np.double)
    if (sigma > 0).any():
        # add zero smoothing for multichannel dimension
        sigma = list(sigma) + [0]
        image = ndi.gaussian_filter(image, sigma)

    if multichannel and (convert2lab or convert2lab is None):
        if image.shape[-1] != 3 and convert2lab:
            raise ValueError("Lab colorspace conversion requires a RGB image.")
        elif image.shape[-1] == 3:
            image = rgb2lab(image)

    if multifeat:
        feat_scale = float(image.shape[3])
    else:
        feat_scale = 1.0

    depth, height, width = image.shape[:3]

    if mask is None:
        mask = np.ones(image.shape[:3], dtype=np.bool)
    else:
        mask = np.asarray(mask, dtype=np.bool)

    if mask.ndim == 2:
        mask = mask[np.newaxis, ...]

    if seed_type == 'nplace':

        segments, step_x, step_y, step_z = place_seed_points(image, mask, n_segments, spacing)

    elif seed_type == 'grid':

        # initialize cluster centroids for desired number of segments
        # essentially just outputs the indices of a grid in the x, y and z direction
        grid_z, grid_y, grid_x = np.mgrid[:depth, :height, :width]
        # returns 3 slices (an object representing an array of slices, see builtin slice)
        slices = regular_grid(image.shape[:3], n_segments)
        step_z, step_y, step_x = [int(s.step) for s in slices]  # extract step size from slices
        segments_z = grid_z[slices]  # use slices to extract coordinates for centre points
        segments_y = grid_y[slices]
        segments_x = grid_x[slices]

        # list of all locations as well as zeros for the color features
        segments_color = np.zeros(segments_z.shape + (image.shape[3],))
        segments = np.concatenate([segments_z[..., np.newaxis],
                                   segments_y[..., np.newaxis],
                                   segments_x[..., np.newaxis],
                                   segments_color],
                                  axis=-1).reshape(-1, 3 + image.shape[3])

        if mask is not None:
            ind1 = mask[segments[:, 0].astype('int'), segments[:, 1].astype('int'), segments[:, 2].astype('int')]
            segments = segments[ind1, :]
    else:
        raise ValueError('seed_type should be nplace or grid')

    segments = np.ascontiguousarray(segments)

    # We do the scaling of ratio in the same way as in the SLIC paper
    # so the values have the same meaning
    step = float(max((step_z, step_y, step_x)))
    ratio = 1.0 / compactness

    image = np.ascontiguousarray(image * ratio, dtype=np.double)
    mask = np.ascontiguousarray(mask, dtype=np.int32)

    #segments_old = np.copy(segments)

    if recompute_seeds:
        # Seed step 2: Run SLIC to reinitialise seeds
        # Runs the supervoxel method but only uses distance to better initialise the method
        labels = _slic_cython(image, mask, segments, step, max_iter, spacing, slic_zero, feat_scale, only_dist=True)

    labels = _slic_cython(image, mask, segments, step, max_iter, spacing, slic_zero, feat_scale, only_dist=False)

    if enforce_connectivity:
        segment_size = mask.sum() / n_segments

        min_size = int(min_size_factor * segment_size)
        max_size = int(max_size_factor * segment_size)

        labels = _enforce_label_connectivity_cython(labels, mask, n_segments, min_size, max_size)

    ret = []
    if is_2d:
        ret.append(labels[0])
    else:
        ret.append(labels)

    if return_adjacency:
        # Also return adjacency map
        labels = np.ascontiguousarray(labels, dtype=np.int32)
        if mask is None:
            adj_mat, border_mat = _find_adjacency_map(labels)
        else:
            adj_mat, border_mat = _find_adjacency_map_mask(labels)

        ret.append(adj_mat)
        ret.append(border_mat)

    if return_segments:
        ret.append(segments)

    if len(ret) == 1:
        return ret[0]
    else:
        return tuple(ret)



"""
maskSLIC: Module for normalising data, e.g. prior to feature extraction
"""

from __future__ import division, print_function

import numpy as np

def norm_percentile(data, percentile=90):
    """
    Normalise the data by dividing by a given percentile

    :param data: Numpy array whose last dimension is assumed to be
                 the volume sequence
    """
    norm = np.percentile(data, percentile)
    return data / norm

def norm_median(data, volume_idx=None):
    """
    Normalise the data by dividing by the median of a given volume

    :param data: Numpy array whose last dimension is assumed to be
                 the volume sequence
    """
    if volume_idx is None:
        volume_idx = data.shape[-1] / 2
    return data / np.median(data[:, volume_idx])

def norm_indiv(data):
    """
    Scale each volume individually so it lies between 0 and 1

    :param data: Numpy array whose last dimension is assumed to be
                 the volume sequence
    """
    data = data - np.min(data, axis=0)
    return data / (np.max(data, axis=0) + 0.001)

def norm_sigenh(data, nvols=3):
    """
    Scale each data point by dividing by a 'baseline' value and then subtracting 1

    This results in 'signal enhancement' curves, starting at 0

    :param data: Numpy array whose last dimension is assumed to be
                 the volume sequence
    """
    data_nvols = data.shape[-1]
    tile_shape = [1, ] * data.ndim
    tile_shape[-1] = nvols

    baseline = np.mean(data[..., :min(nvols, data_nvols)], axis=-1)
    baseline_expanded = np.expand_dims(baseline, axis=-1)
    return data / (baseline_expanded + 0.001) - 1

def normalise(data, method, **kwargs):
    """
    Normalise data using named method

    :param data: Numpy array containing data to be normalised
    :param method: One of ``perc``, ``median``, ``indiv``, or ``sigenh``
    :return: Normalised data as matching Numpy array
    """
    norm_methods = {
        "perc" : norm_percentile,
        "median" : norm_median,
        "indiv" : norm_indiv,
        "sigenh" : norm_sigenh,
    }
    if method in norm_methods:
        return norm_methods[method](data, **kwargs)
    else:
        raise ValueError("Unknown normalisation method: %s" % method)



"""
maskSLIC: Wrapper for supervoxel generation which handles 4D perfusion images using PCA reduction

Copyright (C) 2016-2019, Benjamin Irving
"""

from __future__ import division, print_function, unicode_literals

import numpy as np

from .feat_pca import PcaFeatReduce
from . import slic

def preprocess_pca(img, ncomp, normalise_input_image=True, norm_type='perc'):
    """
    Do PCA analysis on 4D image to turn it into a 3D image suitable for SLIC
    """
    # Normalize enhancement curves using first 3 points (FIXME magic number)
    baseline = np.mean(img[:, :, :, :3], axis=-1)
    img = img - np.tile(np.expand_dims(baseline, axis=-1), (1, 1, 1, img.shape[-1]))

    # Run PCA feature extraction
    pca = PcaFeatReduce(n_components=ncomp, norm_modes=True, norm_input=normalise_input_image, norm_type=norm_type)
    feat_image = pca.get_training_features(img, smooth_timeseries=2.0, feature_volume=True)
    return feat_image

def perfslic(img, roi,
             n_supervoxels=None,
             segment_size=None,
             compactness=0.1,
             seed_type='nplace',
             n_pca_components=3,
             **kwargs):
    """
    Just a wrapper for the SLIC interface
    :param compactness:
    :param sigma:
    :param seed_type: 'grid', 'nrandom'
                    Type of seed point initiliasation which is either based on a grid of no points or randomly
                    assigned within an roi
    :param segment_size: Mean supervoxel size (assuming no ROI but can be used with an ROI as standard)
    :param recompute_seeds: True or False
                            Recompute the initialisation points based on spatial distance within a ROI
    :param n_random_seeds:
    :return:
    """

    if img.ndim > 3 and img.shape[3] > 1:
        # For 4D data, use PCA to reduce down to 3D
        img = preprocess_pca(img, n_pca_components)
    else:
        # For 3D data scale to a range of 0-1
        if img.ndim > 3:
            img = np.squeeze(img, -1)
        img = img.astype(np.float32)
        img = (img - img.min()) / (img.max() - img.min())

    if n_supervoxels is None and segment_size is not None:
        n_supervoxels = int(img.size / segment_size)
    elif n_supervoxels is None:
        raise ValueError("Either the number of supervoxels or the segment size must be given")

    # FIXME enforce_connectivity=True does not seem to work in ROI mode?
    return slic(img,
                mask=roi,
                n_segments=n_supervoxels,
                compactness=compactness,
                seed_type=seed_type,
                multichannel=kwargs.pop("multichannel", False),
                multifeat=kwargs.pop("multifeat", True),
                recompute_seeds=kwargs.pop("recompute_seeds", True),
                **kwargs)



"""
maskSLIC: Simple linear iterative clustering (SLIC) in a region of interest (ROI)

Copyright (C) 2016-2019, Benjamin Irving
"""
from .slic_superpixels import slic
from .perfslic import perfslic
from ._version import __version__, __timestamp__

__all__ = ['slic', 'perfslic', '__version__', '__timestamp__']



"""
maskSLIC: PCA implementation
"""

from __future__ import division, print_function, absolute_import

import logging

import numpy as np

from sklearn.decomposition import PCA
from scipy.ndimage.filters import gaussian_filter1d

from . import normalisation as norm

def _flat_data_to_image(data, roi):
    # return features in image format
    image_shape = list(roi.shape)
    if data.ndim == 2:
        image_shape.append(data.shape[1])
    image = np.zeros(image_shape)
    image[roi] = data
    return image

class PcaFeatReduce(object):
    """
    Extract PCA features from 4D image data

    Thin wrapper around sklearn.decomposition.PCA
    """

    def __init__(self, n_components, norm_modes=True, norm_input=False, norm_type='perc'):
        """
        :param data: 4D image to be processed
        :param roi: Optional sub region to extract
        """
        # Variables
        self.log = logging.getLogger(__name__)
        self.pca = PCA(n_components=n_components)
        self.norm_modes = norm_modes
        self.norm_input = norm_input
        self.norm_type = norm_type

    def get_training_features(self, data, roi=None, smooth_timeseries=None, feature_volume=False):
        """
        Train PCA reduction on data and return features for each voxel

        :param data: 4D data set
        :param roi: Optional 3D ROI
        :param smooth_timeseries: Optional sigma for 1D Gaussian smoothing of each voxel timeseries
        :param feature_volume: determines whether the features are returned as a list or an image

        :return: If ``feature_volume``, 4D array with the same 3d dimensions as data and
                 4th dimension=number of PCA components.
                 Otherwise, 2D array whose first dimension is unmasked voxels and 2nd dimension
                 is the PCA components
        """
        data_inmask, roi = self._mask(data, roi, smooth_timeseries)

        self.log.debug("Using PCA dimensionality reduction")
        reduced_data = self.pca.fit_transform(data_inmask)
        self.log.debug("Number of components: %i", reduced_data.shape[1])

        if self.norm_modes:
            self.log.debug("Normalising PCA modes between 0 and 1")
            reduced_data = norm.normalise(reduced_data, "indiv")

        if not feature_volume:
            return reduced_data

        return _flat_data_to_image(reduced_data, roi)

    def get_projected_test_features(self, data, roi=None, smooth_timeseries=None, feature_volume=False):
        """
        Return features for each voxel from previously trained PCA modes

        :param data: 4D data set
        :param roi: Optional 3D ROI
        :param feature_volume: determines whether the features are returned as a list or an image

        :return: If ``feature_volume``, 4D array with the same 3d dimensions as data and
                 4th dimension=number of PCA components.
                 Otherwise, 2D array whose first dimension is unmasked voxels and 2nd dimension
                 is the PCA components
        """
        data_inmask, roi = self._mask(data, roi, smooth_timeseries)

        #Projecting the data using training set PCA
        if data_inmask.shape[1] != self.pca.mean_.shape[0]:
            raise ValueError("Input data length does not match previous training data")

        reduced_data = self.pca.transform(data_inmask)

        # Scaling features
        if self.norm_modes:
            self.log.debug("Normalising PCA modes")
            reduced_data = norm.normalise(reduced_data, "indiv")

        if not feature_volume:
            return reduced_data

        return _flat_data_to_image(reduced_data, roi)

    def explained_variance(self, cumulative=False):
        """
        Return the variance explained by including each of the modes cumulatively

        :param cumulative: If True, variance explained will be cumulative (i.e.
                           variance explained by all modes up to and including each
                           mode)
        :return: Sequence of numbers giving variance explained for each component.
        """
        if cumulative:
            return [np.sum(self.pca.explained_variance_ratio_[:n+1])
                    for n in range(self.pca.n_components_)]
        else:
            return self.pca.explained_variance_ratio_

    def modes(self):
        """
        :return: Fitted PCA modes
        """
        #return norm.normalise(self.pca.components_, "indiv")
        return [comp + self.pca.mean_ for comp in self.pca.components_]

    def mean(self):
        """
        :return: Training data mean
        """
        #return np.squeeze(norm.normalise(np.expand_dims(self.pca.mean_, axis=0), "indiv"))
        return self.pca.mean_

    def _mask(self, data, roi, smooth_timeseries):
        if roi is None:
            roi = np.ones(data.shape[0:-1], dtype=bool)
        else:
            roi = np.array(roi, dtype=np.bool)
        data_inmask = data[roi]

        if self.norm_input:
            data_inmask = norm.normalise(data_inmask, self.norm_type)

        if smooth_timeseries is not None:
            data_inmask = gaussian_filter1d(data_inmask, sigma=smooth_timeseries, axis=-1)
        return data_inmask, roi




"""
Setup script for maskslic
"""
import os
import subprocess
import re
import io
import glob
import sys

from setuptools import setup
from setuptools import find_packages
from setuptools.extension import Extension

from Cython.Build import cythonize

import numpy

MODULE = 'maskslic'
MODULE_DIR = os.path.abspath(os.path.dirname(__file__))

def get_filetext(rootdir, filename):
    """ Get the text of a local file """
    with io.open(os.path.join(rootdir, filename), encoding='utf-8') as fhandle:
        return fhandle.read()

def git_version():
    """ Get the full and python standardized version from Git tags (if possible) """
    try:
        # Full version includes the Git commit hash
        full_version = subprocess.check_output('git describe --dirty', shell=True).decode("utf-8").strip(" \n")

        # Python standardized version in form major.minor.patch.post<build>
        version_regex = re.compile(r"v?(\d+\.\d+(\.\d+)?(-\d+)?).*")
        match = version_regex.match(full_version)
        if match:
            std_version = match.group(1).replace("-", ".post")
        else:
            raise RuntimeError("Failed to parse version string %s" % full_version)
        return full_version, std_version
    except:
        # Any failure, return None. We may not be in a Git repo at all
        return None, None

def git_timestamp():
    """ Get the last commit timestamp from Git (if possible)"""
    try:
        return subprocess.check_output('git log -1 --format=%cd', shell=True).decode("utf-8").strip(" \n")
    except:
        # Any failure, return None. We may not be in a Git repo at all
        return None

def update_metadata(rootdir, version_str, timestamp_str):
    """ Update the version and timestamp metadata in the module _version.py file """
    with io.open(os.path.join(rootdir, MODULE, "_version.py"), "w", encoding='utf-8') as fhandle:
        fhandle.write("__version__ = '%s'\n" % version_str)
        fhandle.write("__timestamp__ = '%s'\n" % timestamp_str)

def get_requirements(rootdir):
    """ Get a list of all entries in the requirements file """
    with io.open(os.path.join(rootdir, 'requirements.txt'), encoding='utf-8') as fhandle:
        return [l.strip() for l in fhandle.readlines()]

def get_version(rootdir):
    """ Get the current version number (and update it in the module _version.py file if necessary)"""
    version, timestamp = git_version()[1], git_timestamp()

    if version is not None and timestamp is not None:
        # We got the metadata from Git - update the version file
        update_metadata(rootdir, version, timestamp)
    else:
        # Could not get metadata from Git - use the version file if it exists
        with io.open(os.path.join(rootdir, MODULE, '_version.py'), encoding='utf-8') as fhandle:
            metadata = fhandle.read()
            match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", metadata, re.M)
            if match:
                version = match.group(1)
            else:
                version = "unknown"
    return version

def get_package_data(_rootdir):
    """
    Get extra data files to install into the package, e.g. icons
    """
    return {
        MODULE : glob.glob("%s/*.png" % MODULE) + glob.glob("%s/*.svg" % MODULE)
    }

def get_extensions(_rootdir):
    """
    Get Cython extensions
    """
    compile_args = []
    link_args = []

    if sys.platform.startswith('win'):
        compile_args.append('/EHsc')
    elif sys.platform.startswith('darwin'):
        compile_args += ["-mmacosx-version-min=10.9"]
        link_args += ["-stdlib=libc++", "-mmacosx-version-min=10.9"]

    #extensions.append(Extension("%s.perfusionslic.additional.bspline_smoothing" % MODULE,
    #                            sources=["%s/perfusionslic/additional/bspline_smoothing.pyx" % MODULE],
    #                            include_dirs=[numpy.get_include()]))

    #extensions.append(Extension("%s.perfusionslic.additional.create_im" % MODULE,
    #                            sources=["%s/perfusionslic/additional/create_im.pyx" % MODULE],
    #                            include_dirs=[numpy.get_include()]))

    extensions = [
        Extension("%s._slic" % MODULE,
                  sources=["%s/_slic.pyx" % MODULE],
                  include_dirs=[numpy.get_include()]),

        Extension("%s.processing" % MODULE,
                  sources=["%s/processing.pyx" % MODULE,
                           "src/processing.cpp"],
                  include_dirs=["src", numpy.get_include()],
                  language="c++",
                  extra_compile_args=compile_args, extra_link_args=link_args),
    ]

    return cythonize(extensions)

KWARGS = {
    'name' : 'maskslic',
    'version' : get_version(MODULE_DIR),
    'description' : 'Simple linear iterative clustering (SLIC) in a region of interest (ROI)',
    'long_description' : get_filetext(MODULE_DIR, 'README.md'),
    'long_description_content_type' : 'text/markdown',
    'url' : 'https://www.birving.com',
    'author' : 'Benjamin Irving',
    'author_email' : 'mail@birving.com',
    'license' : 'Copyright (C) 2016-2019, Benjamin Irving. See LICENSE file for distribution conditions',
    'setup_requires' : ['cython'],
    'install_requires' : get_requirements(MODULE_DIR),
    'packages' : find_packages(),
    'package_data' : get_package_data(MODULE_DIR),
    'ext_modules' : get_extensions(MODULE_DIR),
    'include_package_data' : True,
    'classifiers' : [
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Science/Research',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
        'License :: Free for non-commercial use',
    ],
}

setup(**KWARGS)
